const chai = require('chai');
const chaiHttp = require('chai-http');
const app = require('../index'); // Adjust the path based on your project structure
const expect = chai.expect;
const User = require('../models/User');
const mongoose = require('mongoose');

chai.use(chaiHttp);
let testUserToken;
describe('Authentication Routes', () => {
    describe('POST /auth', () => {
        it('should create a new user', (done) => {
            chai
                .request(app)
                .post('/user')
                .send({ username: 'usertest', password: 'testpassword', isAgent: false })
                .end((err, res) => {
                    expect(res).to.have.status(200);
                    expect(res.body).to.have.property('message').to.equal('Utilisateur créé avec succès');
                    expect(res.body).to.have.property('user').to.be.an('object');
                    done();
                });
        });
    });

    describe('POST /user/login', () => {
        it('should login and return a token', (done) => {
            chai
                .request(app)
                .post('/user/login')
                .send({ username: 'usertest', password: 'testpassword' })
                .end((err, res) => {
                    expect(res).to.have.status(200);
                    expect(res.body).to.have.property('token').to.be.a('string');
                    done();
                });
        });

        it('should return 401 for invalid login credentials', (done) => {
            chai
                .request(app)
                .post('/user/login')
                .send({ username: 'invaliduser', password: 'invalidpassword' })
                .end((err, res) => {
                    expect(res).to.have.status(401);
                    done();
                });
        });
    });

    describe('POST /user/logout', () => {
        it('should logout successfully', (done) => {
            chai
                .request(app)
                .post('/user/login')
                .send({ username: 'usertest', password: 'testpassword' })
                .end((err, res) => {
                    expect(res).to.have.status(200);
                    const token = res.body.token;
                    chai
                        .request(app)
                        .post('/user/logout')
                        .set('Authorization', `Bearer ${token}`)
                        .end((err, res) => {
                            expect(res).to.have.status(200);
                            expect(res.text).to.equal('Logout successful.');
                            done();
                        });
                });
        });

        it('should return 401 for unauthorized logout attempt', (done) => {
            chai
                .request(app)
                .post('/user/logout')
                .end((err, res) => {
                    expect(res).to.have.status(401);
                    done();
                });
        });
    });
    after(async () => {
        try {
            // Implement logic to delete the test user using Mongoose
            await User.deleteMany({ username: 'usertest' });
        } catch (error) {
            console.error('Error deleting test user:', error);
        } finally {
            await mongoose.connection.close();
        }
    });

});
